// Fetch-on-demand map from local to global text numbers.
//
// $Id: text-mapping.h,v 1.11 1999/11/14 12:54:37 ceder Exp $
// Copyright (C) 1994  Per Cederqvist and Inge Wallin
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef KOMXX_TEXT_MAPPING_H
#define KOMXX_TEXT_MAPPING_H

#include <vector>

#include "kom-types.h"
#include "misc.h"

class get_map_question;
class Local2Global;
class Conference;

class Text_mapping {
  public:
    Text_mapping();		// Creates an invalid object.  Use
				// valid() to test validity.

    Text_mapping(Conference c, int chunk_size=50);
    Text_mapping(const Text_mapping&);
    ~Text_mapping();

    Text_mapping& operator=(const Text_mapping&);

    bool valid() const;

#if 0
    Local_text_no end_no();
    Local_text_no begin_no();
#endif

    Text_no operator [](Local_text_no lno);
    // Prefetch FIRST.
    Status prefetch(Local_text_no first);
    void add(Local_text_no key, Text_no text);
    void add(const Local2Global& l2g);
    void add_zero_range(Local_text_no begin, Local_text_no end);
    int chunk_size() const;

  private:
    struct TMI;
    TMI *tmi;
    
    class Pending {
      public:
	Pending(Local_text_no f, get_map_question *qp)
	  : first(f), q(qp) {}
	
	Local_text_no first;
	get_map_question *q;

    };

    // Find the prefetch entry for lno, creating one if needed.  The
    // iterator is only valid until anything modifies tmi->pending.
    vector<Pending>::iterator get_prefetch(Local_text_no lno);

    void poll_pending();	// Deal with outstanding questions.
};
#endif
